@@ -5,6 +5,7 @@ Tests for `attr._make`.
 from __future__ import absolute_import, division, print_function
 
 import inspect
+import sys
 
 from operator import attrgetter
 
@@ -13,17 +14,19 @@ import pytest
 from hypothesis import given
 from hypothesis.strategies import booleans, integers, lists, sampled_from, text
 
+import attr
+
 from attr import _config
 from attr._compat import PY2
 from attr._make import (
     Attribute,
     Factory,
     _AndValidator,
+    _Attributes,
+    _ClassBuilder,
     _CountingAttr,
     _transform_attrs,
     and_,
-    attr,
-    attributes,
     fields,
     make_class,
     validate,
@@ -33,7 +36,7 @@ from attr.exceptions import NotAnAttrsClassError, DefaultAlreadySetError
 from .utils import (gen_attr_names, list_of_attrs, simple_attr, simple_attrs,
                     simple_attrs_without_metadata, simple_classes)
 
-attrs = simple_attrs.map(lambda c: Attribute.from_counting_attr("name", c))
+attrs_st = simple_attrs.map(lambda c: Attribute.from_counting_attr("name", c))
 
 
 class TestCountingAttr(object):
@@ -44,18 +47,10 @@ class TestCountingAttr(object):
         """
         Returns an instance of _CountingAttr.
         """
-        a = attr()
+        a = attr.ib()
 
         assert isinstance(a, _CountingAttr)
 
-    def _test_validator_none_is_empty_tuple(self):
-        """
-        If validator is None, it's transformed into an empty tuple.
-        """
-        a = attr(validator=None)
-
-        assert () == a._validator
-
     def test_validators_lists_to_wrapped_tuples(self):
         """
         If a list is passed as validator, it's just converted to a tuple.
@@ -66,7 +61,7 @@ class TestCountingAttr(object):
         def v2(_, __):
             pass
 
-        a = attr(validator=[v1, v2])
+        a = attr.ib(validator=[v1, v2])
 
         assert _AndValidator((v1, v2,)) == a._validator
 
@@ -75,7 +70,7 @@ class TestCountingAttr(object):
         If _CountingAttr.validator is used as a decorator and there is no
         decorator set, the decorated method is used as the validator.
         """
-        a = attr()
+        a = attr.ib()
 
         @a.validator
         def v():
@@ -97,7 +92,7 @@ class TestCountingAttr(object):
         def v(_, __):
             pass
 
-        a = attr(validator=wrap(v))
+        a = attr.ib(validator=wrap(v))
 
         @a.validator
         def v2(self, _, __):
@@ -110,7 +105,7 @@ class TestCountingAttr(object):
         Raise DefaultAlreadySetError if the decorator is used after a default
         has been set.
         """
-        a = attr(default=42)
+        a = attr.ib(default=42)
 
         with pytest.raises(DefaultAlreadySetError):
             @a.default
@@ -122,7 +117,7 @@ class TestCountingAttr(object):
         Decorator wraps the method in a Factory with pass_self=True and sets
         the default.
         """
-        a = attr()
+        a = attr.ib()
 
         @a.default
         def f(self):
@@ -133,9 +128,9 @@ class TestCountingAttr(object):
 
 def make_tc():
     class TransformC(object):
-        z = attr()
-        y = attr()
-        x = attr()
+        z = attr.ib()
+        y = attr.ib()
+        x = attr.ib()
         a = 42
     return TransformC
 
@@ -144,39 +139,44 @@ class TestTransformAttrs(object):
     """
     Tests for `_transform_attrs`.
     """
+    def test_no_modifications(self):
+        """
+        Doesn't attach __attrs_attrs__ to the class anymore.
+        """
+        C = make_tc()
+        _transform_attrs(C, None, False)
+
+        assert None is getattr(C, "__attrs_attrs__", None)
+
     def test_normal(self):
         """
         Transforms every `_CountingAttr` and leaves others (a) be.
         """
         C = make_tc()
-        _transform_attrs(C, None)
-        assert ["z", "y", "x"] == [a.name for a in C.__attrs_attrs__]
+        attrs, _, = _transform_attrs(C, None, False)
+
+        assert ["z", "y", "x"] == [a.name for a in attrs]
 
     def test_empty(self):
         """
         No attributes works as expected.
         """
-        @attributes
+        @attr.s
         class C(object):
             pass
 
-        _transform_attrs(C, None)
+        assert _Attributes(((), [])) == _transform_attrs(C, None, False)
 
-        assert () == C.__attrs_attrs__
-
-    @pytest.mark.parametrize("attribute", [
-        "z",
-        "y",
-        "x",
-    ])
-    def test_transforms_to_attribute(self, attribute):
+    def test_transforms_to_attribute(self):
         """
         All `_CountingAttr`s are transformed into `Attribute`s.
         """
         C = make_tc()
-        _transform_attrs(C, None)
+        attrs, super_attrs = _transform_attrs(C, None, False)
 
-        assert isinstance(getattr(C, attribute), Attribute)
+        assert [] == super_attrs
+        assert 3 == len(attrs)
+        assert all(isinstance(a, Attribute) for a in attrs)
 
     def test_conflicting_defaults(self):
         """
@@ -184,65 +184,35 @@ class TestTransformAttrs(object):
         mandatory attributes.
         """
         class C(object):
-            x = attr(default=None)
-            y = attr()
+            x = attr.ib(default=None)
+            y = attr.ib()
 
         with pytest.raises(ValueError) as e:
-            _transform_attrs(C, None)
+            _transform_attrs(C, None, False)
         assert (
             "No mandatory attributes allowed after an attribute with a "
             "default value or factory.  Attribute in question: Attribute"
             "(name='y', default=NOTHING, validator=None, repr=True, "
             "cmp=True, hash=None, init=True, convert=None, "
-            "metadata=mappingproxy({}))",
+            "metadata=mappingproxy({}), type=None)",
         ) == e.value.args
 
     def test_these(self):
         """
-        If these is passed, use it and ignore body.
+        If these is passed, use it and ignore body and super classes.
         """
-        class C(object):
-            y = attr()
+        class Base(object):
+            z = attr.ib()
 
-        _transform_attrs(C, {"x": attr()})
+        class C(Base):
+            y = attr.ib()
+
+        attrs, super_attrs = _transform_attrs(C, {"x": attr.ib()}, False)
+
+        assert [] == super_attrs
         assert (
             simple_attr("x"),
-        ) == C.__attrs_attrs__
-        assert isinstance(C.y, _CountingAttr)
-
-    def test_recurse(self):
-        """
-        Collect attributes from all sub-classes.
-        """
-        class A(object):
-            a = None
-
-        class B(A):
-            b = attr()
-
-        _transform_attrs(B, None)
-
-        class C(B):
-            c = attr()
-
-        _transform_attrs(C, None)
-
-        class D(C):
-            d = attr()
-
-        _transform_attrs(D, None)
-
-        class E(D):
-            e = attr()
-
-        _transform_attrs(E, None)
-
-        assert (
-            simple_attr("b"),
-            simple_attr("c"),
-            simple_attr("d"),
-            simple_attr("e"),
-        ) == E.__attrs_attrs__
+        ) == attrs
 
 
 class TestAttributes(object):
@@ -255,18 +225,20 @@ class TestAttributes(object):
         Raises TypeError on old-style classes.
         """
         with pytest.raises(TypeError) as e:
-            @attributes
+            @attr.s
             class C:
                 pass
+
         assert ("attrs only works with new-style classes.",) == e.value.args
 
     def test_sets_attrs(self):
         """
         Sets the `__attrs_attrs__` class attribute with a list of `Attribute`s.
         """
-        @attributes
+        @attr.s
         class C(object):
-            x = attr()
+            x = attr.ib()
+
         assert "x" == C.__attrs_attrs__[0].name
         assert all(isinstance(a, Attribute) for a in C.__attrs_attrs__)
 
@@ -274,13 +246,14 @@ class TestAttributes(object):
         """
         No attributes, no problems.
         """
-        @attributes
+        @attr.s
         class C3(object):
             pass
+
         assert "C3()" == repr(C3())
         assert C3() == C3()
 
-    @given(attr=attrs, attr_name=sampled_from(Attribute.__slots__))
+    @given(attr=attrs_st, attr_name=sampled_from(Attribute.__slots__))
     def test_immutable(self, attr, attr_name):
         """
         Attribute instances are immutable.
@@ -304,11 +277,11 @@ class TestAttributes(object):
         sentinel = object()
 
         class C(object):
-            x = attr()
+            x = attr.ib()
 
         setattr(C, method_name, sentinel)
 
-        C = attributes(C)
+        C = attr.s(C)
         meth = getattr(C, method_name)
 
         assert sentinel != meth
@@ -338,11 +311,11 @@ class TestAttributes(object):
         am_args[arg_name] = False
 
         class C(object):
-            x = attr()
+            x = attr.ib()
 
         setattr(C, method_name, sentinel)
 
-        C = attributes(**am_args)(C)
+        C = attr.s(**am_args)(C)
 
         assert sentinel == getattr(C, method_name)
 
@@ -352,9 +325,9 @@ class TestAttributes(object):
         """
         On Python 3, the name in repr is the __qualname__.
         """
-        @attributes(slots=slots_outer)
+        @attr.s(slots=slots_outer)
         class C(object):
-            @attributes(slots=slots_inner)
+            @attr.s(slots=slots_inner)
             class D(object):
                 pass
 
@@ -366,9 +339,9 @@ class TestAttributes(object):
         """
         Setting repr_ns overrides a potentially guessed namespace.
         """
-        @attributes(slots=slots_outer)
+        @attr.s(slots=slots_outer)
         class C(object):
-            @attributes(repr_ns="C", slots=slots_inner)
+            @attr.s(repr_ns="C", slots=slots_inner)
             class D(object):
                 pass
         assert "C.D()" == repr(C.D())
@@ -379,9 +352,9 @@ class TestAttributes(object):
         """
         On Python 3, __name__ is different from __qualname__.
         """
-        @attributes(slots=slots_outer)
+        @attr.s(slots=slots_outer)
         class C(object):
-            @attributes(slots=slots_inner)
+            @attr.s(slots=slots_inner)
             class D(object):
                 pass
 
@@ -395,21 +368,49 @@ class TestAttributes(object):
         """
         monkeypatch.setattr(_config, "_run_validators", with_validation)
 
-        @attributes
+        @attr.s
         class C(object):
-            x = attr()
-            y = attr()
+            x = attr.ib()
+            y = attr.ib()
 
             def __attrs_post_init__(self2):
                 self2.z = self2.x + self2.y
 
         c = C(x=10, y=20)
+
         assert 30 == getattr(c, 'z', None)
 
+    def test_types(self):
+        """
+        Sets the `Attribute.type` attr from type argument.
+        """
+        @attr.s
+        class C(object):
+            x = attr.ib(type=int)
+            y = attr.ib(type=str)
+            z = attr.ib()
 
-@attributes
+        assert int is fields(C).x.type
+        assert str is fields(C).y.type
+        assert None is fields(C).z.type
+
+    @pytest.mark.parametrize("slots", [True, False])
+    def test_clean_class(self, slots):
+        """
+        Attribute definitions do not appear on the class body after @attr.s.
+        """
+        @attr.s(slots=slots)
+        class C(object):
+            x = attr.ib()
+
+        x = getattr(C, "x", None)
+
+        assert not isinstance(x, _CountingAttr)
+
+
+@attr.s
 class GC(object):
-    @attributes
+    @attr.s
     class D(object):
         pass
 
@@ -428,10 +429,10 @@ class TestMakeClass(object):
         """
         C1 = make_class("C1", ls(["a", "b"]))
 
-        @attributes
+        @attr.s
         class C2(object):
-            a = attr()
-            b = attr()
+            a = attr.ib()
+            b = attr.ib()
 
         assert C1.__attrs_attrs__ == C2.__attrs_attrs__
 
@@ -439,12 +440,15 @@ class TestMakeClass(object):
         """
         Passing a dict of name: _CountingAttr creates an equivalent class.
         """
-        C1 = make_class("C1", {"a": attr(default=42), "b": attr(default=None)})
+        C1 = make_class("C1", {
+            "a": attr.ib(default=42),
+            "b": attr.ib(default=None),
+        })
 
-        @attributes
+        @attr.s
         class C2(object):
-            a = attr(default=42)
-            b = attr(default=None)
+            a = attr.ib(default=42)
+            b = attr.ib(default=None)
 
         assert C1.__attrs_attrs__ == C2.__attrs_attrs__
 
@@ -453,7 +457,8 @@ class TestMakeClass(object):
         attributes_arguments are passed to attributes
         """
         C = make_class("C", ["x"], repr=False)
-        assert repr(C(1)).startswith("<attr._make.C object at 0x")
+
+        assert repr(C(1)).startswith("<tests.test_make.C object at 0x")
 
     def test_catches_wrong_attrs_type(self):
         """
@@ -474,12 +479,34 @@ class TestMakeClass(object):
             pass
 
         cls = make_class("C", {})
+
         assert cls.__mro__[-1] == object
 
         cls = make_class("C", {}, bases=(D,))
+
         assert D in cls.__mro__
         assert isinstance(cls(), D)
 
+    @pytest.mark.parametrize("slots", [True, False])
+    def test_clean_class(self, slots):
+        """
+        Attribute definitions do not appear on the class body.
+        """
+        C = make_class("C", ["x"], slots=slots)
+
+        x = getattr(C, "x", None)
+
+        assert not isinstance(x, _CountingAttr)
+
+    def test_missing_sys_getframe(self, monkeypatch):
+        """
+        `make_class()` does not fail when `sys._getframe()` is not available.
+        """
+        monkeypatch.delattr(sys, '_getframe')
+        C = make_class("C", ["x"])
+
+        assert 1 == len(C.__attrs_attrs__)
+
 
 class TestFields(object):
     """
@@ -491,6 +518,7 @@ class TestFields(object):
         """
         with pytest.raises(TypeError) as e:
             fields(C(1, 2))
+
         assert "Passed object must be a class." == e.value.args[0]
 
     def test_handler_non_attrs_class(self, C):
@@ -527,9 +555,12 @@ class TestConvert(object):
         """
         Return value of convert is used as the attribute's value.
         """
-        C = make_class("C", {"x": attr(convert=lambda v: v + 1),
-                             "y": attr()})
+        C = make_class("C", {
+            "x": attr.ib(convert=lambda v: v + 1),
+            "y": attr.ib(),
+        })
         c = C(1, 2)
+
         assert c.x == 2
         assert c.y == 2
 
@@ -538,11 +569,12 @@ class TestConvert(object):
         """
         Property tests for attributes with convert.
         """
-        C = make_class("C", {"y": attr(),
-                             "x": attr(init=init, default=val,
-                                       convert=lambda v: v + 1),
-                             })
+        C = make_class("C", {
+            "y": attr.ib(),
+            "x": attr.ib(init=init, default=val, convert=lambda v: v + 1),
+        })
         c = C(2)
+
         assert c.x == val + 1
         assert c.y == 2
 
@@ -551,12 +583,15 @@ class TestConvert(object):
         """
         Property tests for attributes with convert, and a factory default.
         """
-        C = make_class("C", {"y": attr(),
-                             "x": attr(init=init,
-                                       default=Factory(lambda: val),
-                                       convert=lambda v: v + 1),
-                             })
+        C = make_class("C", {
+            "y": attr.ib(),
+            "x": attr.ib(
+                init=init,
+                default=Factory(lambda: val),
+                convert=lambda v: v + 1),
+        })
         c = C(2)
+
         assert c.x == val + 1
         assert c.y == 2
 
@@ -564,14 +599,22 @@ class TestConvert(object):
         """
         If takes_self on factories is True, self is passed.
         """
-        C = make_class("C", {"x": attr(default=Factory(
-            (lambda self: self), takes_self=True
-        ))})
+        C = make_class("C", {
+            "x": attr.ib(
+                default=Factory((lambda self: self), takes_self=True)
+            ),
+        })
 
         i = C()
 
         assert i is i.x
 
+    def test_factory_hashable(self):
+        """
+        Factory is hashable.
+        """
+        assert hash(Factory(None, False)) == hash(Factory(None, False))
+
     def test_convert_before_validate(self):
         """
         Validation happens after conversion.
@@ -579,9 +622,10 @@ class TestConvert(object):
         def validator(inst, attr, val):
             raise RuntimeError("foo")
         C = make_class(
-            "C",
-            {"x": attr(validator=validator, convert=lambda v: 1 / 0),
-             "y": attr()})
+            "C", {
+                "x": attr.ib(validator=validator, convert=lambda v: 1 / 0),
+                "y": attr.ib(),
+            })
         with pytest.raises(ZeroDivisionError):
             C(1, 2)
 
@@ -589,7 +633,9 @@ class TestConvert(object):
         """
         Converters circumvent immutability.
         """
-        C = make_class("C", {"x": attr(convert=lambda v: int(v))}, frozen=True)
+        C = make_class("C", {
+            "x": attr.ib(convert=lambda v: int(v)),
+        }, frozen=True)
         C("1")
 
 
@@ -599,10 +645,12 @@ class TestValidate(object):
     """
     def test_success(self):
         """
-        If the validator suceeds, nothing gets raised.
+        If the validator succeeds, nothing gets raised.
         """
-        C = make_class("C", {"x": attr(validator=lambda *a: None),
-                             "y": attr()})
+        C = make_class("C", {
+            "x": attr.ib(validator=lambda *a: None),
+            "y": attr.ib()
+        })
         validate(C(1, 2))
 
     def test_propagates(self):
@@ -613,7 +661,7 @@ class TestValidate(object):
             if value == 42:
                 raise FloatingPointError
 
-        C = make_class("C", {"x": attr(validator=raiser)})
+        C = make_class("C", {"x": attr.ib(validator=raiser)})
         i = C(1)
         i.x = 42
 
@@ -630,7 +678,7 @@ class TestValidate(object):
         def raiser(_, __, ___):
             raise Exception(obj)
 
-        C = make_class("C", {"x": attr(validator=raiser)})
+        C = make_class("C", {"x": attr.ib(validator=raiser)})
         c = C(1)
         validate(c)
         assert 1 == c.x
@@ -656,7 +704,7 @@ class TestValidate(object):
             if value == 42:
                 raise ValueError("omg")
 
-        C = make_class("C", {"x": attr(validator=[v1, v2])})
+        C = make_class("C", {"x": attr.ib(validator=[v1, v2])})
 
         validate(C(1))
 
@@ -674,8 +722,8 @@ class TestValidate(object):
         """
         Empty list/tuple for validator is the same as None.
         """
-        C1 = make_class("C", {"x": attr(validator=[])})
-        C2 = make_class("C", {"x": attr(validator=None)})
+        C1 = make_class("C", {"x": attr.ib(validator=[])})
+        C2 = make_class("C", {"x": attr.ib(validator=None)})
 
         assert inspect.getsource(C1.__init__) == inspect.getsource(C2.__init__)
 
@@ -744,3 +792,45 @@ class TestMetadata(object):
         C = make_class("C", dict(zip(gen_attr_names(), list_of_attrs)))
         for a in fields(C)[1:]:
             assert a.metadata is fields(C)[0].metadata
+
+
+class TestClassBuilder(object):
+    """
+    Tests for `_ClassBuilder`.
+    """
+    def test_repr_str(self):
+        """
+        Trying to add a `__str__` without having a `__repr__` raises a
+        ValueError.
+        """
+        with pytest.raises(ValueError) as ei:
+            make_class("C", {}, repr=False, str=True)
+
+        assert (
+            "__str__ can only be generated if a __repr__ exists.",
+        ) == ei.value.args
+
+    def test_repr(self):
+        """
+        repr of builder itself makes sense.
+        """
+        class C(object):
+            pass
+
+        b = _ClassBuilder(C, None, True, True, False)
+
+        assert "<_ClassBuilder(cls=C)>" == repr(b)
+
+    def test_returns_self(self):
+        """
+        All methods return the builder for chaining.
+        """
+        class C(object):
+            x = attr.ib()
+
+        b = _ClassBuilder(C, None, True, True, False)
+
+        cls = b.add_cmp().add_hash().add_init().add_repr("ns").add_str() \
+            .build_class()
+
+        assert "ns.C(x=1)" == repr(cls(1))
